﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using EVCB_OCPP.Domain;
using EVCB_OCPP.Domain.Models.Database;
using EVCB_OCPP.Packet.Features;
using EVCB_OCPP.Packet.Messages;
using EVCB_OCPP.Packet.Messages.Core;
using EVCB_OCPP.Packet.Messages.SubTypes;
using EVCB_OCPP.WSServer.Dto;
using Newtonsoft.Json;
using OCPPPackage.Profiles;
using OCPPServer.Protocol;

namespace EVCB_OCPP.WSServer.Message
{

    internal partial class ProfileHandler
    {
        private List<string> idtag_RejecttionList = new List<string>() { "AC5CDC3D", "48C4CBD9257A" };
        int counnter = 0;
        internal MessageResult ExecuteCoreRequest(Actions action, ClientData session, IRequest request)
        {
            MessageResult result = new MessageResult() { Success = false };

            switch (action)
            {

                case Actions.BootNotification:
                    {
                        BootNotificationRequest _request = request as BootNotificationRequest;
                        int heartbeat_interval = GlobalConfig.GetHEARTBEAT_INTERVAL();
                        using (var db = new MainDBContext())
                        {
                            var _machine = db.Machine.Where(x => x.ChargeBoxId == session.ChargeBoxId).FirstOrDefault();
                            _machine.ChargeBoxSerialNumber = string.IsNullOrEmpty(_request.chargeBoxSerialNumber) ? string.Empty : _request.chargeBoxSerialNumber;
                            _machine.ChargePointModel = string.IsNullOrEmpty(_request.chargePointModel) ? string.Empty : _request.chargePointModel;
                            _machine.ChargePointVendor = string.IsNullOrEmpty(_request.chargePointVendor) ? string.Empty : _request.chargePointVendor;
                            _machine.FW_CurrentVersion = string.IsNullOrEmpty(_request.firmwareVersion) ? string.Empty : _request.firmwareVersion;
                            _machine.Iccid = string.IsNullOrEmpty(_request.iccid) ? string.Empty : _request.iccid;
                            _machine.Imsi = string.IsNullOrEmpty(_request.imsi) ? string.Empty : _request.imsi;
                            _machine.MeterSerialNumber = string.IsNullOrEmpty(_request.meterSerialNumber) ? string.Empty : _request.meterSerialNumber;
                            _machine.MeterType = string.IsNullOrEmpty(_request.meterType) ? string.Empty : _request.meterType;

                            db.SaveChanges();

                            var configVaule = db.MachineConfigure.Where(x => x.ChargeBoxId == session.ChargeBoxId && x.ConfigureName == StandardConfiguration.HeartbeatInterval)
                                .Select(x => x.ConfigureSetting).FirstOrDefault();

                            if (configVaule != null)
                            {
                                int.TryParse(configVaule, out heartbeat_interval);
                            }
                        }
                        var confirm = new BootNotificationConfirmation() { currentTime = DateTime.Now, interval = heartbeat_interval, status = Packet.Messages.SubTypes.RegistrationStatus.Accepted };

                        result.Message = confirm;
                        result.Success = true;
                    }
                    break;
                case Actions.StatusNotification:
                    {
                        //只保留最新上報狀況
                        StatusNotificationRequest _request = request as StatusNotificationRequest;

                        {
                            List<string> removeIds = new List<string>();
                            ConnectorErrorStauts _oldStatus = null;
                            using (var oldDB = new MainDBContext())
                            {
                                if (_request.status == Packet.Messages.SubTypes.ChargePointStatus.Faulted)
                                {
                                    _oldStatus = oldDB.ConnectorStatus.Where(x => x.ChargeBoxId == session.ChargeBoxId
                                 && x.ConnectorId == _request.connectorId).Select(x => new ConnectorErrorStauts
                                 {
                                     Status = x.Status,
                                     ChargePointErrorCodeId = x.ChargePointErrorCodeId,
                                     ErrorInfo = x.ErrorInfo,
                                     VendorId = x.VendorId,
                                     VendorErrorCode = x.VendorErrorCode
                                 }).FirstOrDefault();
                                }

                                removeIds = oldDB.ConnectorStatus.Where(x => x.ChargeBoxId == session.ChargeBoxId
                                 && x.ConnectorId == _request.connectorId).Select(x => x.Id).ToList();
                            }

                            using (var db = new MainDBContext())
                            {
                                foreach (var id in removeIds)
                                {

                                    string query = "DELETE FROM [dbo].[ConnectorStatus] WHERE [Id]={0} ";
                                    db.Database.ExecuteSqlCommand(query, id);
                                }

                                db.SaveChanges();
                                var _currentStatus = new Domain.Models.Database.ConnectorStatus()
                                {
                                    ChargeBoxId = session.ChargeBoxId,
                                    ConnectorId = (byte)_request.connectorId,
                                    CreatedOn = _request.timestamp.HasValue ? _request.timestamp.Value : DateTime.Now,
                                    Status = (int)_request.status,
                                    ChargePointErrorCodeId = (int)_request.errorCode,
                                    ErrorInfo = _request.info,
                                    VendorId = _request.vendorId,
                                    VendorErrorCode = _request.vendorErrorCode,
                                    Id = Guid.NewGuid().ToString()
                                };
                                db.ConnectorStatus.Add(_currentStatus);

                                if (_request.status == Packet.Messages.SubTypes.ChargePointStatus.Faulted)
                                {
                                    db.MachineError.Add(new MachineError()
                                    {
                                        ConnectorId = (byte)_request.connectorId,
                                        CreatedOn = _request.timestamp.HasValue ? _request.timestamp.Value : DateTime.Now,
                                        Status = (int)_request.status,
                                        ChargeBoxId = session.ChargeBoxId,
                                        ErrorCodeId = (int)_request.errorCode,
                                        ErrorInfo = _request.info,
                                        PreStatus = _oldStatus.Status,
                                        VendorErrorCode = _request.vendorErrorCode,
                                        VendorId = _request.vendorId
                                    });
                                }


                                db.SaveChanges();
                            }
                        }

                        var confirm = new StatusNotificationConfirmation() { };
                        result.Message = confirm;
                        result.Success = true;
                    }
                    break;
                case Actions.Heartbeat:
                    {
                        using (var db = new MainDBContext())
                        {
                            var machine = db.Machine.Where(x => x.ChargeBoxId == session.ChargeBoxId).FirstOrDefault();
                            if (machine != null)
                            {
                                db.Configuration.AutoDetectChangesEnabled = false;
                                db.Configuration.ValidateOnSaveEnabled = false;
                                db.Machine.Attach(machine);
                                machine.HeartbeatUpdatedOn = DateTime.Now;

                                db.Entry(machine).Property(x => x.HeartbeatUpdatedOn).IsModified = true;
                                db.SaveChanges();
                            }
                            var confirm = new HeartbeatConfirmation() { currentTime = DateTime.Now };
                            result.Message = confirm;
                            result.Success = true;
                        }
                    }
                    break;
                case Actions.MeterValues:
                    {
                        MeterValuesRequest _request = request as MeterValuesRequest;

                        if (_request.meterValue.Count > 0)
                        {
                            using (var db = new MeterValueDBContext())
                            {
                                foreach (var item in _request.meterValue)
                                {
                                    foreach (var sampleVaule in item.sampledValue)
                                    {

                                        string sp = "[dbo].[uspInsertMeterValueRecord] @ChargeBoxId," +
                     "@ConnectorId,@Value,@CreatedOn,@ContextId,@FormatId,@MeasurandId,@PhaseId,@LocationId,@UnitId,@TransactionId";

                                        List<SqlParameter> parameter = new List<SqlParameter>
                                        {
                                              new SqlParameter("ChargeBoxId",session.ChargeBoxId),
                                              new SqlParameter("ConnectorId",  (byte)_request.connectorId),
                                              new SqlParameter("Value",sampleVaule.value),
                                              new SqlParameter("CreatedOn",item.timestamp),
                                              new SqlParameter("ContextId",sampleVaule.context.HasValue ? (int)sampleVaule.context : 0),
                                              new SqlParameter("FormatId",sampleVaule.format.HasValue ? (int)sampleVaule.format : 0),
                                              new SqlParameter("MeasurandId",sampleVaule.measurand.HasValue ? (int)sampleVaule.measurand : 0),
                                              new SqlParameter("PhaseId",sampleVaule.phase.HasValue ? (int)sampleVaule.phase : 0),
                                              new SqlParameter("LocationId",sampleVaule.location.HasValue ? (int)sampleVaule.location : 0),
                                              new SqlParameter("UnitId",sampleVaule.unit.HasValue ? (int)sampleVaule.unit : 0),
                                              new SqlParameter("TransactionId",_request.transactionId.HasValue?_request.transactionId:-1),
                                          };


                                        db.Database.ExecuteSqlCommand(sp, parameter.ToArray());
                                    }
                                }
                            }
                        }

                        var confirm = new MeterValuesConfirmation() { };
                        result.Message = confirm;
                        result.Success = true;
                    }
                    break;
                case Actions.StartTransaction:
                    {
                        StartTransactionRequest _request = request as StartTransactionRequest;
                        long _pastTime = DateTimeOffset.Now.AddYears(-20).ToUnixTimeSeconds();
                        int _tranId = (int)(_pastTime % 100000000) * 10 + 1;


                        //需處理StopTransactionOnInvalidId 情境，時間若和當前時間差距10秒，發動Authorize詢問後在回覆
                        using (var db = new MainDBContext())
                        {
                            var _CustomerId = db.Machine.Where(x => x.ChargeBoxId == session.ChargeBoxId).Include(x => x.Customer).
                                 Select(x => x.CustomerId).FirstOrDefault();

                            db.TransactionRecord.Add(new TransactionRecord()
                            {
                                ChargeBoxId = session.ChargeBoxId,
                                ConnectorId = (byte)_request.connectorId,
                                CreatedOn = DateTime.Now,
                                StartIdTag = _request.idTag,
                                MeterStart = _request.meterStart,
                                CustomerId = _CustomerId,
                                StartTime = _request.timestamp,
                                ReservationId = _request.reservationId.HasValue ? _request.reservationId.Value : 0,
                                TransactionId = _tranId

                            });

                            db.SaveChanges();
                        }



                        var confirm = new StartTransactionConfirmation()
                        {
                            idTagInfo = new Packet.Messages.SubTypes.IdTagInfo()
                            {
                                status = idtag_RejecttionList.Contains(_request.idTag) ? AuthorizationStatus.Invalid : AuthorizationStatus.Accepted
                            },
                            transactionId = _tranId



                        };


                        result.Message = confirm;
                        result.Success = true;
                    }
                    break;
                case Actions.StopTransaction:
                    {
                        StopTransactionRequest _request = request as StopTransactionRequest;
                        List<TransactionDataRecord> _TransactionDatas = new List<TransactionDataRecord>();
                        if (_request.transactionData != null)
                        {
                            if (_request.transactionData.Count > 0)
                            {
                                foreach (var item in _request.transactionData)
                                {
                                    foreach (var sampleVaule in item.sampledValue)
                                    {
                                        _TransactionDatas.Add(new TransactionDataRecord()
                                        {
                                            ChargeBoxId = session.ChargeBoxId,
                                            MeasuredOn = item.timestamp,
                                            TransactionId = _request.transactionId,
                                            ContextId = sampleVaule.context.HasValue ? (int)sampleVaule.context : 0,
                                            FormatId = sampleVaule.format.HasValue ? (int)sampleVaule.format : 0,
                                            LocationId = sampleVaule.location.HasValue ? (int)sampleVaule.location : 0,
                                            MeasurandId = sampleVaule.measurand.HasValue ? (int)sampleVaule.measurand : 0,
                                            PhaseId = sampleVaule.phase.HasValue ? (int)sampleVaule.phase : 0,
                                            UnitId = sampleVaule.unit.HasValue ? (int)sampleVaule.unit : 0,
                                            Value = sampleVaule.value
                                        });
                                    }
                                }

                            }
                        }

                        using (var db = new MainDBContext())
                        {
                            var transaction = db.TransactionRecord.Where(x => x.TransactionId == _request.transactionId
                             && x.ChargeBoxId == session.ChargeBoxId).FirstOrDefault();

                            if (transaction != null)
                            {
                                transaction.MeterStop = _request.meterStop;
                                transaction.StopTime = _request.timestamp;
                                transaction.StopReasonId = _request.reason.HasValue ? (int)_request.reason.Value : 0;
                                transaction.StopIdTag = _request.idTag;

                                if (_TransactionDatas.Count > 0)
                                {
                                    _TransactionDatas.ForEach(x => x.ConnectorId = (byte)transaction.ConnectorId);
                                }

                                db.SaveChanges();
                                var confirm = new StopTransactionConfirmation()
                                {
                                    idTagInfo = new Packet.Messages.SubTypes.IdTagInfo()
                                    {

                                        status = Packet.Messages.SubTypes.AuthorizationStatus.Accepted
                                    }

                                };
                                result.Message = confirm;
                                result.Success = true;
                            }
                            else
                            {
                                result.Exception = new Exception("Can't find transactionId " + _request.transactionId);

                            }
                        }
                    }
                    break;
                case Actions.Authorize:
                    {
                        AuthorizeRequest _request = request as AuthorizeRequest;

                        var confirm = new AuthorizeConfirmation()
                        {

                            idTagInfo = new Packet.Messages.SubTypes.IdTagInfo()
                            {
                                status = idtag_RejecttionList.Contains(_request.idTag) ? AuthorizationStatus.Invalid : AuthorizationStatus.Accepted
                            }
                        };
                        result.Message = confirm;
                        result.Success = true;
                    }
                    break;
                default:
                    {
                        Console.WriteLine(string.Format("Not Implement {0} Logic", request.GetType().ToString().Replace("OCPPPackage.Messages.Core.", "")));
                    }
                    break;
            }
            return result;
        }

        internal MessageResult ExecuteCoreConfirm(Actions action, ClientData session, IConfirmation confirm, string requestId)
        {
            MessageResult result = new MessageResult() { Success = true };

            switch (action)
            {
                case Actions.ChangeAvailability:
                    {
                        ChangeAvailabilityConfirmation _confirm = confirm as ChangeAvailabilityConfirmation;
                        ChangeAvailabilityRequest _request = _confirm.GetRequest() as ChangeAvailabilityRequest;

                        using (var db = new MainDBContext())
                        {
                            var operation = db.MachineOperateRecord.Where(x => x.SerialNo == requestId &&
                            x.ChargeBoxId == session.ChargeBoxId && x.Status == 0).FirstOrDefault();
                            if (operation != null)
                            {
                                operation.FinishedOn = DateTime.Now;
                                operation.Status = 1;//電樁有回覆
                                operation.EVSE_Status = (int)_confirm.status;
                                operation.EVSE_Value = _confirm.status.ToString();
                                db.SaveChanges();
                            }

                        }
                    }
                    break;

                case Actions.ClearCache:
                    {
                        ClearCacheConfirmation _confirm = confirm as ClearCacheConfirmation;
                        ClearCacheRequest _request = _confirm.GetRequest() as ClearCacheRequest;

                        using (var db = new MainDBContext())
                        {
                            var operation = db.MachineOperateRecord.Where(x => x.SerialNo == requestId &&
                            x.ChargeBoxId == session.ChargeBoxId && x.Status == 0).FirstOrDefault();
                            if (operation != null)
                            {
                                operation.FinishedOn = DateTime.Now;
                                operation.Status = 1;//電樁有回覆
                                operation.EVSE_Status = (int)_confirm.status;
                                operation.EVSE_Value = _confirm.status.ToString();
                                db.SaveChanges();
                            }

                        }
                    }
                    break;
                case Actions.RemoteStartTransaction:
                    {
                        RemoteStartTransactionConfirmation _confirm = confirm as RemoteStartTransactionConfirmation;
                        RemoteStartTransactionRequest _request = _confirm.GetRequest() as RemoteStartTransactionRequest;

                        using (var db = new MainDBContext())
                        {
                            var operation = db.MachineOperateRecord.Where(x => x.SerialNo == requestId &&
                            x.ChargeBoxId == session.ChargeBoxId && x.Status == 0).FirstOrDefault();
                            if (operation != null)
                            {
                                operation.FinishedOn = DateTime.Now;
                                operation.Status = 1;//電樁有回覆
                                operation.EVSE_Status = (int)_confirm.status;
                                operation.EVSE_Value = _confirm.status.ToString();
                                db.SaveChanges();
                            }

                        }
                    }
                    break;
                case Actions.RemoteStopTransaction:
                    {
                        RemoteStopTransactionConfirmation _confirm = confirm as RemoteStopTransactionConfirmation;
                        RemoteStopTransactionRequest _request = _confirm.GetRequest() as RemoteStopTransactionRequest;

                        using (var db = new MainDBContext())
                        {
                            var operation = db.MachineOperateRecord.Where(x => x.SerialNo == requestId &&
                            x.ChargeBoxId == session.ChargeBoxId && x.Status == 0).FirstOrDefault();
                            if (operation != null)
                            {
                                operation.FinishedOn = DateTime.Now;
                                operation.Status = 1;//電樁有回覆
                                operation.EVSE_Status = (int)_confirm.status;
                                operation.EVSE_Value = _confirm.status.ToString();
                                db.SaveChanges();
                            }

                        }
                    }
                    break;
                case Actions.Reset:
                    {
                        ResetConfirmation _confirm = confirm as ResetConfirmation;
                        ResetRequest _request = _confirm.GetRequest() as ResetRequest;

                        using (var db = new MainDBContext())
                        {
                            var operation = db.MachineOperateRecord.Where(x => x.SerialNo == requestId &&
                            x.ChargeBoxId == session.ChargeBoxId && x.Status == 0).FirstOrDefault();
                            if (operation != null)
                            {
                                operation.FinishedOn = DateTime.Now;
                                operation.Status = 1;//電樁有回覆
                                operation.EVSE_Status = (int)_confirm.status;
                                operation.EVSE_Value = _confirm.status.ToString();
                                db.SaveChanges();
                            }

                        }
                    }
                    break;
                case Actions.ChangeConfiguration:
                    {
                        ChangeConfigurationConfirmation _confirm = confirm as ChangeConfigurationConfirmation;
                        ChangeConfigurationRequest _request = _confirm.GetRequest() as ChangeConfigurationRequest;

                        using (var db = new MainDBContext())
                        {
                            var operation = db.MachineOperateRecord.Where(x => x.SerialNo == requestId &&
                     x.ChargeBoxId == session.ChargeBoxId && x.Status == 0).FirstOrDefault();

                            if (operation != null)
                            {
                                operation.FinishedOn = DateTime.Now;
                                operation.Status = 1;//電樁有回覆
                                operation.EVSE_Status = (int)_confirm.status;
                                operation.EVSE_Value = _confirm.status.ToString();

                            }

                            if (_confirm.status == Packet.Messages.SubTypes.ConfigurationStatus.Accepted || _confirm.status == Packet.Messages.SubTypes.ConfigurationStatus.RebootRequired)
                            {
                                var configure = db.MachineConfigure.Where(x => x.ChargeBoxId == session.ChargeBoxId).ToList();

                                var foundConfig = configure.Find(x => x.ConfigureName == _request.key);
                                if (foundConfig != null)
                                {
                                    foundConfig.ReadOnly = false;
                                    foundConfig.ConfigureSetting = _request.value;
                                }
                                else
                                {
                                    db.MachineConfigure.Add(new MachineConfigure()
                                    {
                                        ChargeBoxId = session.ChargeBoxId,
                                        ConfigureName = _request.key,
                                        ReadOnly = false,
                                        ConfigureSetting = _request.value
                                    });
                                }
                            }
                            db.SaveChanges();
                        }

                    }
                    break;
                case Actions.GetConfiguration:
                    {
                        GetConfigurationConfirmation _confirm = confirm as GetConfigurationConfirmation;
                        GetConfigurationRequest _request = _confirm.GetRequest() as GetConfigurationRequest;

                        using (var db = new MainDBContext())
                        {
                            var configure = db.MachineConfigure.Where(x => x.ChargeBoxId == session.ChargeBoxId).ToList();

                            if (_confirm.configurationKey != null)
                            {
                                foreach (var item in _confirm.configurationKey)
                                {
                                    var foundConfig = configure.Find(x => x.ConfigureName == item.key);
                                    if (foundConfig != null)
                                    {
                                        foundConfig.ReadOnly = item.IsReadOnly;
                                        foundConfig.ConfigureSetting = item.value;
                                    }
                                    else
                                    {
                                        db.MachineConfigure.Add(new MachineConfigure()
                                        {
                                            ChargeBoxId = session.ChargeBoxId,
                                            ConfigureName = item.key,
                                            ReadOnly = item.IsReadOnly,
                                            ConfigureSetting = item.value,
                                            Exists=true
                                        });
                                    }
                                }
                            }
                            if (_confirm.unknownKey != null)
                            {
                                foreach (var item in _confirm.unknownKey)
                                {
                                    var foundConfig = configure.Find(x => x.ConfigureName == item);
                                    if (foundConfig != null)
                                    {
                                        foundConfig.ReadOnly = true;
                                        foundConfig.ConfigureSetting = string.Empty;
                                        foundConfig.Exists = false;
                                    }
                                    else
                                    {
                                        db.MachineConfigure.Add(new MachineConfigure()
                                        {
                                            ChargeBoxId = session.ChargeBoxId,
                                            ConfigureName = item
                                        });
                                    }
                                }
                            }




                            var operation = db.MachineOperateRecord.Where(x => x.SerialNo == requestId &&
                           x.ChargeBoxId == session.ChargeBoxId && x.Status == 0).FirstOrDefault();

                            if (operation != null)
                            {
                                operation.FinishedOn = DateTime.Now;
                                operation.Status = 1;//電樁有回覆
                                operation.EVSE_Status = 1;
                                operation.EVSE_Value = JsonConvert.SerializeObject(_confirm.configurationKey, Formatting.None);

                            }

                            db.SaveChanges();

                        }
                    }
                    break;
                default:
                    {
                        Console.WriteLine(string.Format("Not Implement {0} Logic", confirm.GetType().ToString().Replace("OCPPPackage.Messages.Core.", "")));
                    }
                    break;
            }
            return result;
        }


        internal MessageResult ReceivedCoreError(Actions action, string errorMsg, ClientData session, string requestId)
        {
            MessageResult result = new MessageResult() { Success = true };

            switch (action)
            {
                case Actions.ChangeAvailability:
                case Actions.ChangeConfiguration:
                case Actions.ClearCache:
                case Actions.RemoteStartTransaction:
                case Actions.RemoteStopTransaction:
                case Actions.Reset:
                case Actions.GetConfiguration:
                    {
                        using (var db = new MainDBContext())
                        {
                            var operation = db.MachineOperateRecord.Where(x => x.SerialNo == requestId &&
                            x.ChargeBoxId == session.ChargeBoxId && x.Status == 0).FirstOrDefault();
                            if (operation != null)
                            {
                                operation.FinishedOn = DateTime.Now;
                                operation.Status = 1;//電樁有回覆
                                operation.EVSE_Status = (int)255;//錯誤
                                operation.EVSE_Value = errorMsg;
                                db.SaveChanges();
                            }

                        }

                    }
                    break;

                default:
                    {
                        Console.WriteLine(string.Format("Not Implement {0} Logic", action));
                    }
                    break;
            }
            return result;

        }
    }
}
